@@ -13,7 +13,7 @@
 
   COPYRIGHT:
 
-    (c) 2011-2015, martin isenburg, rapidlasso - fast tools to catch reality
+    (c) 2011-2017, martin isenburg, rapidlasso - fast tools to catch reality
 
     This is free software; you can redistribute and/or modify it under the
     terms of the GNU Lesser General Licence as published by the Free Software
@@ -24,9 +24,6 @@
 
   CHANGE HISTORY:
 
-    24 December 2016 -- by Jean-Romain Roussel -- L53 changed && by || to enable compiling with clang++ on Linux
-    20 December 2016 -- by Jean-Romain Roussel -- Change fprint(stderr, ...), raise an exeption
-
     see corresponding header file
 
 ===============================================================================
@@ -36,7 +33,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <stdexcept>
 
 #include "lasquadtree.hpp"
 #include "lasinterval.hpp"
@@ -49,15 +45,19 @@
 #include "bytestreamout_file.hpp"
 
 #ifdef UNORDERED
-  // Check if on OS X and using cland (unordered map isn't part of tr1 namespace)
-  #if defined(__APPLE__) || defined(__clang__)
-    #include <unordered_map>
-    using namespace std;
-  #else
-    #include <tr1/unordered_map>
+   // Figure out whether <unordered_map> is in tr1
+#  ifdef __has_include
+#    if __has_include(<unordered_map>)
+#     include <unordered_map>
+      using namespace std;
+#     define UNORDERED_FOUND
+#    endif
+#  endif
+#  ifndef UNORDERED_FOUND
+#    include <tr1/unordered_map>
     using namespace std;
     using namespace tr1;
-  #endif
+#   endif
 typedef unordered_map<I32,U32> my_cell_hash;
 #else
 #include <hash_map>
@@ -101,7 +101,7 @@ void LASindex::complete(U32 minimum_points, I32 maximum_intervals, const BOOL ve
 {
   if (verbose)
   {
-    throw std::runtime_error(std::string("before complete %d %d")); //minimum_points, maximum_intervals
+    REprintf("before complete %d %d\n", minimum_points, maximum_intervals);
     print(FALSE);
   }
   if (minimum_points)
@@ -173,7 +173,7 @@ void LASindex::complete(U32 minimum_points, I32 maximum_intervals, const BOOL ve
     }
     if (verbose)
     {
-      throw std::runtime_error(std::string("after minimum_points %d")); //minimum_points
+      REprintf("after minimum_points %d\n", minimum_points);
       print(FALSE);
     }
   }
@@ -186,7 +186,7 @@ void LASindex::complete(U32 minimum_points, I32 maximum_intervals, const BOOL ve
     interval->merge_intervals(maximum_intervals, verbose);
     if (verbose)
     {
-      throw std::runtime_error(std::string("after maximum_intervals %d")); //maximum_intervals
+      REprintf("after maximum_intervals %d\n", maximum_intervals);
       print(FALSE);
     }
   }
@@ -212,15 +212,15 @@ void LASindex::print(BOOL verbose)
     }
     if (total_check != interval->total)
     {
-      throw std::runtime_error(std::string("ERROR: total_check %d != interval->total %d")); //total_check, interval->total
+      REprintf("ERROR: total_check %d != interval->total %d\n", total_check, interval->total);
     }
-    if (verbose) throw std::runtime_error(std::string("cell %d intervals %d full %d total %d (%.2f)")); //interval->index, intervals, interval->full, interval->total, 100.0f*interval->full/interval->total
+    if (verbose) REprintf("cell %d intervals %d full %d total %d (%.2f)\n", interval->index, intervals, interval->full, interval->total, 100.0f*interval->full/interval->total);
     total_cells++;
     total_full += interval->full;
     total_total += interval->total;
     total_intervals += intervals;
   }
-  if (verbose) throw std::runtime_error(std::string("total cells/intervals %d/%d full %d (%.2f)")); //total_cells, total_intervals, total_full, 100.0f*total_full/total_total
+  if (verbose) REprintf("total cells/intervals %d/%d full %d (%.2f)\n", total_cells, total_intervals, total_full, 100.0f*total_full/total_total);
 }
 
 LASquadtree* LASindex::get_spatial() const
@@ -237,7 +237,7 @@ BOOL LASindex::intersect_rectangle(const F64 r_min_x, const F64 r_min_y, const F
 {
   have_interval = FALSE;
   cells = spatial->intersect_rectangle(r_min_x, r_min_y, r_max_x, r_max_y);
-//  throw std::runtime_error(std::string("%d cells of %g/%g %g/%g intersect rect %g/%g %g/%g")); //num_cells, spatial->get_min_x(), spatial->get_min_y(), spatial->get_max_x(), spatial->get_max_y(), r_min_x, r_min_y, r_max_x, r_max_y
+//  REprintf("%d cells of %g/%g %g/%g intersect rect %g/%g %g/%g\n", num_cells, spatial->get_min_x(), spatial->get_min_y(), spatial->get_max_x(), spatial->get_max_y(), r_min_x, r_min_y, r_max_x, r_max_y);
   if (cells)
     return merge_intervals();
   return FALSE;
@@ -247,7 +247,7 @@ BOOL LASindex::intersect_tile(const F32 ll_x, const F32 ll_y, const F32 size)
 {
   have_interval = FALSE;
   cells = spatial->intersect_tile(ll_x, ll_y, size);
-//  throw std::runtime_error(std::string("%d cells of %g/%g %g/%g intersect tile %g/%g/%g")); //num_cells, spatial->get_min_x(), spatial->get_min_y(), spatial->get_max_x(), spatial->get_max_y(), ll_x, ll_y, size
+//  REprintf("%d cells of %g/%g %g/%g intersect tile %g/%g/%g\n", num_cells, spatial->get_min_x(), spatial->get_min_y(), spatial->get_max_x(), spatial->get_max_y(), ll_x, ll_y, size);
   if (cells)
     return merge_intervals();
   return FALSE;
@@ -257,7 +257,7 @@ BOOL LASindex::intersect_circle(const F64 center_x, const F64 center_y, const F6
 {
   have_interval = FALSE;
   cells = spatial->intersect_circle(center_x, center_y, radius);
-//  throw std::runtime_error(std::string("%d cells of %g/%g %g/%g intersect circle %g/%g/%g")); //num_cells, spatial->get_min_x(), spatial->get_min_y(), spatial->get_max_x(), spatial->get_max_y(), center_x, center_y, radius
+//  REprintf("%d cells of %g/%g %g/%g intersect circle %g/%g/%g\n", num_cells, spatial->get_min_x(), spatial->get_min_y(), spatial->get_max_x(), spatial->get_max_y(), center_x, center_y, radius);
   if (cells)
     return merge_intervals();
   return FALSE;
@@ -283,6 +283,40 @@ BOOL LASindex::has_intervals()
   return FALSE;
 }
 
+BOOL LASindex::read(FILE* file)
+{
+  if (file == 0) return FALSE;
+  ByteStreamIn* stream;
+  if (IS_LITTLE_ENDIAN())
+    stream = new ByteStreamInFileLE(file);
+  else
+    stream = new ByteStreamInFileBE(file);
+  if (!read(stream))
+  {
+    delete stream;
+    return FALSE;
+  }
+  delete stream;
+  return TRUE;
+}
+
+BOOL LASindex::write(FILE* file) const
+{
+  if (file == 0) return FALSE;
+  ByteStreamOut* stream;
+  if (IS_LITTLE_ENDIAN())
+    stream = new ByteStreamOutFileLE(file);
+  else
+    stream = new ByteStreamOutFileBE(file);
+  if (!write(stream))
+  {
+    delete stream;
+    return FALSE;
+  }
+  delete stream;
+  return TRUE;
+}
+
 BOOL LASindex::read(const char* file_name)
 {
   if (file_name == 0) return FALSE;
@@ -302,25 +336,20 @@ BOOL LASindex::read(const char* file_name)
     name[strlen(name)-1] = 'x';
   }
   FILE* file = fopen(name, "rb");
-  free(name);
   if (file == 0)
   {
+    free(name);
     return FALSE;
   }
-  ByteStreamIn* stream;
-  if (IS_LITTLE_ENDIAN())
-    stream = new ByteStreamInFileLE(file);
-  else
-    stream = new ByteStreamInFileBE(file);
-  if (!read(stream))
+  if (!read(file))
   {
-    throw std::runtime_error(std::string("ERROR (LASindex): cannot read '%s'")); //name
-    delete stream;
+    REprintf("ERROR (LASindex): cannot read '%s'\n", name);
     fclose(file);
+    free(name);
     return FALSE;
   }
-  delete stream;
   fclose(file);
+  free(name);
   return TRUE;
 }
 
@@ -377,7 +406,7 @@ BOOL LASindex::append(const char* file_name) const
       CHAR user_id[16];
       try { bytestreamin->getBytes((U8*)user_id, 16); } catch(...)
       {
-        throw std::runtime_error(std::string("ERROR: reading header.vlrs[%d].user_id")); //u
+        REprintf("ERROR: reading header.vlrs[%d].user_id\n", u);
         return FALSE;
       }
       if (strcmp(user_id, "laszip encoded") == 0)
@@ -388,13 +417,13 @@ BOOL LASindex::append(const char* file_name) const
       U16 record_id;
       try { bytestreamin->get16bitsLE((U8*)&record_id); } catch(...)
       {
-        throw std::runtime_error(std::string("ERROR: reading header.vlrs[%d].record_id")); //u
+        REprintf("ERROR: reading header.vlrs[%d].record_id\n", u);
         return FALSE;
       }
       U16 record_length_after_header;
       try { bytestreamin->get16bitsLE((U8*)&record_length_after_header); } catch(...)
       {
-        throw std::runtime_error(std::string("ERROR: reading header.vlrs[%d].record_length_after_header")); //u
+        REprintf("ERROR: reading header.vlrs[%d].record_length_after_header\n", u);
         return FALSE;
       }
       total += (54 + record_length_after_header);
@@ -427,7 +456,7 @@ BOOL LASindex::append(const char* file_name) const
 
   if (!write(bytestreamout))
   {
-    throw std::runtime_error(std::string("ERROR (LASindex): cannot append LAX to '%s'")); //file_name
+    REprintf("ERROR (LASindex): cannot append LAX to '%s'\n", file_name);
     delete bytestreamout;
     fclose(file);
     delete lasreader;
@@ -484,24 +513,17 @@ BOOL LASindex::write(const char* file_name) const
   FILE* file = fopen(name, "wb");
   if (file == 0)
   {
-    throw std::runtime_error(std::string("ERROR (LASindex): cannot open '%s' for write")); //name
+    REprintf("ERROR (LASindex): cannot open '%s' for write\n", name);
     free(name);
     return FALSE;
   }
-  ByteStreamOut* stream;
-  if (IS_LITTLE_ENDIAN())
-    stream = new ByteStreamOutFileLE(file);
-  else
-    stream = new ByteStreamOutFileBE(file);
-  if (!write(stream))
+  if (!write(file))
   {
-    throw std::runtime_error(std::string("ERROR (LASindex): cannot write '%s'")); //name
-    delete stream;
+    REprintf("ERROR (LASindex): cannot write '%s'\n", name);
     fclose(file);
     free(name);
     return FALSE;
   }
-  delete stream;
   fclose(file);
   free(name);
   return TRUE;
@@ -522,32 +544,32 @@ BOOL LASindex::read(ByteStreamIn* stream)
   char signature[4];
   try { stream->getBytes((U8*)signature, 4); } catch (...)
   {
-    throw std::runtime_error(std::string("ERROR (LASindex): reading signature"));
+    REprintf("ERROR (LASindex): reading signature\n");
     return FALSE;
   }
   if (strncmp(signature, "LASX", 4) != 0)
   {
-    throw std::runtime_error(std::string("ERROR (LASindex): wrong signature %4s instead of 'LASX'")); //signature
+    REprintf("ERROR (LASindex): wrong signature %4s instead of 'LASX'\n", signature);
     return FALSE;
   }
   U32 version;
   try { stream->get32bitsLE((U8*)&version); } catch (...)
   {
-    throw std::runtime_error(std::string("ERROR (LASindex): reading version"));
+    REprintf("ERROR (LASindex): reading version\n");
     return FALSE;
   }
   // read spatial quadtree
   spatial = new LASquadtree();
   if (!spatial->read(stream))
   {
-    throw std::runtime_error(std::string("ERROR (LASindex): cannot read LASspatial (LASquadtree)"));
+    REprintf("ERROR (LASindex): cannot read LASspatial (LASquadtree)\n");
     return FALSE;
   }
   // read interval
   interval = new LASinterval();
   if (!interval->read(stream))
   {
-    throw std::runtime_error(std::string("ERROR (LASindex): reading LASinterval"));
+    REprintf("ERROR (LASindex): reading LASinterval\n");
     return FALSE;
   }
   // tell spatial about the existing cells
@@ -563,25 +585,25 @@ BOOL LASindex::write(ByteStreamOut* stream) const
 {
   if (!stream->putBytes((U8*)"LASX", 4))
   {
-    throw std::runtime_error(std::string("ERROR (LASindex): writing signature"));
+    REprintf("ERROR (LASindex): writing signature\n");
     return FALSE;
   }
   U32 version = 0;
   if (!stream->put32bitsLE((U8*)&version))
   {
-    throw std::runtime_error(std::string("ERROR (LASindex): writing version"));
+    REprintf("ERROR (LASindex): writing version\n");
     return FALSE;
   }
   // write spatial quadtree
   if (!spatial->write(stream))
   {
-    throw std::runtime_error(std::string("ERROR (LASindex): cannot write LASspatial (LASquadtree)"));
+    REprintf("ERROR (LASindex): cannot write LASspatial (LASquadtree)\n");
     return FALSE;
   }
   // write interval
   if (!interval->write(stream))
   {
-    throw std::runtime_error(std::string("ERROR (LASindex): writing LASinterval"));
+    REprintf("ERROR (LASindex): writing LASinterval\n");
     return FALSE;
   }
   return TRUE;
@@ -634,7 +656,7 @@ BOOL LASindex::merge_intervals()
         used_cells++;
       }
     }
-//    throw std::runtime_error(std::string("LASindex: used %d cells of total %d")); //used_cells, interval->get_number_cells()
+//    REprintf("LASindex: used %d cells of total %d\n", used_cells, interval->get_number_cells());
     if (used_cells)
     {
       BOOL r = interval->merge();
